from numpy import float64, int64
import Simplifier

class Monomial:
    def __init__(self, coeff: float64 | int64, exp: int64):
        self.coeff = coeff
        self.exp = exp
        
    def __init__(self):
        self.coeff = 0.0
        self.exp = 0.0
    
    def __str__(self) -> str:
        return f"({self.coeff})X^({self.exp})"
    
    def getCoeff(self) -> float64:
        return self.coeff

    def getExp(self) -> int64:
        return self.exp

    def setCoeff(self, coeff: float64):
        self.coeff = coeff

    def setExp(self, exp: int64) -> None:
        self.exp = exp

    def eval(self, val: float64 | int64) -> float64:
        return self.coeff * pow(val, self.exp)
    eval.__doc__ = "a method which evaluates the monomial where X=val"

    def dy_dx(self) -> Simplifier.Monomial:
        return Monomial(self.coeff*self.exp, self.exp-1)
    dy_dx.__doc__ = "return a Monomial that is the first derivative of itself"



from numpy import float64, int64

class ImaginaryNumber:
    __doc__ = """
        a class to store the value of a complex number in the form a+bi
    """
    def __init__(self):
        self.a = 0.0
        self.b = 0.0
        
    def __init__(self, a: float64 | int64, b: float64 | int64):
        self.a = a
        self.b = b

    def setA(self, a: int64 | float64) -> None:
        self.a = a

    def setB(self, b: int64 | float64) -> None:
        self.b = b
        
    def getA(self) -> float64:
        return self.a

    def getB(self) -> float64:
        return self.b

    def __str__(self) -> str:
        if self.b > 0.0:
            return f"{self.a}+{self.b}i"
        elif self.b < 0.0:
            return f"{self.a}-{self.b * -1.0}i"
        else:
            return ""



from numpy import float64, int64
import Simplifier
from Simplifier.Monomial import Monomial

class Polynomial:

    __doc__ = '''
        A class for a polynomial
    '''

    m = Monomial()
    def __init__(self, *args: Monomial):
        temp = []
        for i in args:
            if type(i) == type(self.m):
                temp.append(i)
        self.list = temp

    def __init__(self):
        self.list = []
    
    def add(self, monomial: Monomial) -> None:
        self.list.append(monomial)
        
    def remove(self, monomial: Monomial) -> None:
        self.list.remove(monomial)
    
    def __str__(self) -> str:
        out = ""
        for i in self.list:
            if i.getCoeff() < 0:
                out = out + f" - {i.getCoeff() * -1}X^{i.getExp()}"
            elif i.getCoeff() > 0:
                out = out + f" + {i.getCoeff()}X^{i.getExp()}"
            else:
                pass
        
        return out
    
    def dy_dx(self) -> Simplifier.Polynomial:
        temp = Polynomial()
        for i in self.list:
            temp.add(i.dy_dx())
        return temp
    dy_dx.__doc__ = 'A method to return the derivative of itself'

    def eval(self, val: float64 | int64) -> float64:
        temp = 0.0
        for i in self.list:
            temp += i.eval(val)
        return temp
    eval.__doc__ = 'A method to the value of the polynomial at X=val'